#!python2.6

import os

import cherrypy

import messages as msg

from genshi.template import TemplateLoader
from genshi.template import Context

from lib import params

templates = None
loader  = None

def init_loader():
    global templates, loader
    templates = os.path.join(params.WEB_DIR, 'html')
    loader = TemplateLoader(templates, auto_reload=True)


def render(context):

	context.setdefault('session', cherrypy.session)
	ctx = Context(**context)
	base = 'base.html'
	tmpl = loader.load(base)
	
	stream = tmpl.generate(ctx)
	return stream.render('html', encoding='utf-8')


def authenticate_admin(name, pwd):
	return True

def admin_auth(func):

	def __auth(*args, **kwargs):

		try:
			request_page = cherrypy.request.request_line.split(' ')[1]
		except IndexError:
			request_page = cherrypy.request.page_info

		if 'admin' in cherrypy.session:
			return func(*args, **kwargs)
		else:
			try:
				admin_name = kwargs['username'].decode('utf-8')
				admin_pass = kwargs['password'].decode('utf-8')
			except KeyError:
				return login_page(request_page, retry_msg="")

			isAuthenticated = authenticate_admin(admin_name, admin_pass)

			if isAuthenticated:
				cherrypy.session['admin'] = admin_name

				return func(*args, **kwargs)
			else:
				return login_page(request_page, retry_msg=msg.INVALID_AUTH)
	
	return __auth


def user_auth(func):

	def __auth(*args, **kwargs):

		try:
			request_page = cherrypy.request.request_line.split(' ')[1]
		except IndexError:
			request_page = cherrypy.request.page_info

		if 'user' in cherrypy.session:
			return func(*args, **kwargs)
		else:
			try:
				username = kwargs['username'].decode('utf-8')
				password = kwargs['password'].decode('utf-8')
			except KeyError:
				return login_page(request_page, retry_msg="")

			isAuthenticated = authenticate_user(username, password)

			if isAuthenticated:
				cherrypy.session['user'] = username

				return func(*args, **kwargs)
			else:
				return login_page(request_page, retry_msg=msg.INVALID_AUTH)
	
	return __auth


def login_page(orig_request, retry_msg):
	return render({
					'err_msg': retry_msg,
					'request': orig_request,
					'page': 'login.html'
					})
